@@ -1,8 +1,10 @@
 """Interactive explorers mostly based on Bokeh."""
 import numpy as np
 from wasabi import msg as logger
+from bokeh.plotting import figure
 from bokeh.models import CustomJS, ColumnDataSource, CDSView, IndexFilter
 from bokeh.layouts import column, row
+from bokeh.palettes import Category20
 from abc import ABC, abstractmethod
 from hover import module_config
 from hover.utils.misc import current_time
@@ -11,7 +13,7 @@ from .local_config import bokeh_hover_tooltip
 
 class BokehForLabeledText(ABC):
     """
-    Base class that keeps template figure settings.
+    Base class that keeps template explorer settings.
     """
 
     DEFAULT_FIGURE_KWARGS = {
@@ -37,29 +39,49 @@ class BokehForLabeledText(ABC):
         "output_backend": "webgl",
     }
 
-    def __init__(self, **kwargs):
+    DATA_KEY_TO_KWARGS = {}
+
+    def __init__(self, df_dict, **kwargs):
         """
         Operations shared by all child classes.
+
         (1) settle the figure settings by using child class defaults + kwargs overrides
-        (2) create a blank figure under such settings
+        (2) settle the glyph settings by using child class defaults
         (3) create widgets that child classes can override
+        (4) create data sources the correspond to class-specific data subsets.
+        (5) activate builtin search callbacks depending on the child class.
+        (6) create a (likely) blank figure under such settings
         """
-        self.figure_settings = self.__class__.DEFAULT_FIGURE_KWARGS.copy()
-        self.figure_settings.update(kwargs)
+        logger.divider(f"Initializing {self.__class__.__name__}")
+        self.figure_kwargs = self.__class__.DEFAULT_FIGURE_KWARGS.copy()
+        self.figure_kwargs.update(kwargs)
+        self.glyph_kwargs = {
+            _key: _dict["constant"].copy()
+            for _key, _dict in self.__class__.DATA_KEY_TO_KWARGS.items()
+        }
+        self._setup_widgets()
+        self._setup_dfs(df_dict)
+        self._setup_sources()
+        self._activate_search_builtin()
         self.reset_figure()
-        self.setup_widgets()
 
     def reset_figure(self):
-        from bokeh.plotting import figure
+        """Start over on the figure."""
+        logger.info("Creating/resetting Figure")
+        if hasattr(self, "figure"):
+            self.figure.renderers.clear()
+        else:
+            self.figure = figure(**self.figure_kwargs)
 
-        self.figure = figure(**self.figure_settings)
-
-    def setup_widgets(self):
+    def _setup_widgets(self):
         """
         Prepare widgets for interactive functionality.
+
+        Create positive/negative text search boxes.
         """
         from bokeh.models import TextInput
 
+        logger.info("Setting up widgets")
         self.search_pos = TextInput(
             title="Text contains (plain text, or /pattern/flag for regex):",
             width_policy="fit",
@@ -70,21 +92,58 @@ class BokehForLabeledText(ABC):
         )
 
     def layout_widgets(self):
-        """
-        Define the layout of widgets.
-        """
+        """Define the layout of widgets."""
         return column(self.search_pos, self.search_neg)
 
     def view(self):
-        """
-        Define the layout of the whole explorer.
-        """
+        """Define the layout of the whole explorer."""
         return column(self.layout_widgets(), self.figure)
 
+    def _setup_dfs(self, df_dict):
+        """
+        Check and store DataFrames.
+
+        Intended to be extended in child classes for pre/post processing.
+        """
+        logger.info("Setting up dfs")
+        expected_keys = set(self.__class__.DATA_KEY_TO_KWARGS.keys())
+        assert (
+            set(df_dict.keys()) == expected_keys
+        ), f"Expected the keys of df_dict to be exactly {expected_keys}"
+
+        self.dfs = {_key: _df.copy() for _key, _df in df_dict.items()}
+
+    def _setup_sources(self):
+        """
+        Create (NOT UPDATE) ColumnDataSource objects.
+
+        Intended to be extended in child classes for pre/post processing.
+        """
+        logger.info("Setting up sources")
+        self.sources = {_key: ColumnDataSource(_df) for _key, _df in self.dfs.items()}
+
+    def _activate_search_builtin(self):
+        """
+        Typically called once during initialization.
+        Highlight positive search results and mute negative search results.
+
+        Note that this is a template method which heavily depends on class attributes.
+        """
+        logger.info("Activating built-in search")
+        for _key, _dict in self.__class__.DATA_KEY_TO_KWARGS.items():
+            for _flag, _params in _dict["search"].items():
+                logger.info(
+                    f"Activated {_flag} on subset {_key} to respond to the search widgets."
+                )
+                self.glyph_kwargs[_key] = self.activate_search(
+                    self.sources[_key], self.glyph_kwargs[_key], altered_param=_params
+                )
+
     def activate_search(self, source, kwargs, altered_param=("size", 10, 5, 7)):
         """
         Enables string/regex search-and-highlight mechanism.
-        Modifies plotting source in-place.
+
+        Modifies the plotting source in-place.
         """
         assert isinstance(source, ColumnDataSource)
         assert isinstance(kwargs, dict)
@@ -177,114 +236,96 @@ class BokehCorpusExplorer(BokehForLabeledText):
     Plot unlabeled, 2-D-vectorized text data points in a corpus.
     """
 
-    BACKGROUND_GLYPH_KWARGS = {
-        "color": "gainsboro",
-        "line_alpha": 0.3,
-        "legend_label": "unlabeled",
+    DATA_KEY_TO_KWARGS = {
+        "raw": {
+            "constant": {"line_alpha": 0.4},
+            "search": {
+                "size": ("size", 10, 5, 7),
+                "fill_alpha": ("fill_alpha", 0.4, 0.1, 0.2),
+                "color": ("color", "coral", "linen", "gainsboro"),
+            },
+        }
     }
 
-    def __init__(self, df_raw, **kwargs):
+    def __init__(self, df_dict, **kwargs):
         """
         Requires the input dataframe to contain:
 
         (1) "x" and "y" columns for coordinates;
         (2) a "text" column for data point tooltips.
         """
-        super().__init__(**kwargs)
+        super().__init__(df_dict, **kwargs)
 
-        # prepare plot-ready dataframe for train set
-        for _key in ["text", "x", "y"]:
-            assert _key in df_raw.columns
-        self.df_raw = df_raw.copy()
+    def _setup_dfs(self, df_dict):
+        """Extending from the parent method."""
+        for _key in self.__class__.DATA_KEY_TO_KWARGS.keys():
+            for _col in ["text", "x", "y"]:
+                assert _col in df_dict[_key].columns
 
-        # plot the train set as a background
-        self.background_kwargs = self.__class__.BACKGROUND_GLYPH_KWARGS.copy()
-        self.source = ColumnDataSource(self.df_raw)
-        self._activate_search_on_corpus()
-
-    def _activate_search_on_corpus(self):
-        """
-        Assuming that there will not be a labeled dev set.
-        """
-        self.background_kwargs = self.activate_search(
-            self.source, self.background_kwargs, altered_param=("size", 10, 5, 7)
-        )
-        self.background_kwargs = self.activate_search(
-            self.source,
-            self.background_kwargs,
-            altered_param=("fill_alpha", 0.6, 0.4, 0.5),
-        )
-        self.background_kwargs = self.activate_search(
-            self.source,
-            self.background_kwargs,
-            altered_param=("color", "coral", "linen", "gainsboro"),
-        )
+        super()._setup_dfs(df_dict)
 
     def plot(self, *args, **kwargs):
         """
         (Re)-plot the corpus.
         Called just once per instance most of the time.
         """
-        self.reset_figure()
-        self.figure.circle("x", "y", source=self.source, **self.background_kwargs)
+        self.figure.circle(
+            "x", "y", source=self.sources["raw"], **self.glyph_kwargs["raw"]
+        )
 
 
-class BokehCorpusAnnotator(BokehForLabeledText):
+class BokehCorpusAnnotator(BokehCorpusExplorer):
     """
     [SERVER ONLY]
     Annoate text data points via callbacks.
     """
 
-    def __init__(self, df_working, **kwargs):
+    DATA_KEY_TO_KWARGS = {
+        "raw": {
+            "constant": {"line_alpha": 0.3},
+            "search": {
+                "size": ("size", 10, 5, 7),
+                "fill_alpha": ("fill_alpha", 0.3, 0.05, 0.2),
+            },
+        }
+    }
+
+    def __init__(self, df_dict, **kwargs):
+        """Conceptually the same as the parent method."""
+        super().__init__(df_dict, **kwargs)
+
+    def _setup_dfs(self, df_dict):
         """
-        Requires the input dataframe to contain:
+        Extending from the parent method.
 
-        (1) "x" and "y" columns for coordinates;
-        (2) a "text" column for data point tooltips.
-
-        Optional:
-        (3) a "label" column that defaults to ABSTAIN.
+        Add a "label" column if it is not present.
         """
-        super().__init__(**kwargs)
+        super()._setup_dfs(df_dict)
 
-        for _key in ["text", "x", "y"]:
-            assert _key in df_working.columns
-        self.df_working = df_working.copy()
-        if not "label" in self.df_working.columns:
-            self.df_working["label"] = module_config.ABSTAIN_DECODED
+        if not "label" in self.dfs["raw"].columns:
+            self.dfs["raw"]["label"] = module_config.ABSTAIN_DECODED
 
-        self.source = ColumnDataSource(self.df_working)
-        self.plot_kwargs = {"line_alpha": 0.3}
-        self.reset_source()
-        self.plot()
-
-    def reset_source(self):
-        self.source.data = self.df_working
-        self.plot_kwargs = self.activate_search(
-            self.source, self.plot_kwargs, altered_param=("size", 10, 5, 7)
-        )
-        self.plot_kwargs = self.activate_search(
-            self.source, self.plot_kwargs, altered_param=("fill_alpha", 0.4, 0.05, 0.2)
-        )
+    def update_source(self):
+        """Note that it seems required to re-activate the search widgets."""
+        self.sources["raw"].data = self.dfs["raw"]
+        self._activate_search_builtin()
 
     def layout_widgets(self):
-        """
-        Define the layout of widgets.
-        """
+        """Define the layout of widgets."""
         first_row = row(self.search_pos, self.search_neg)
         second_row = row(
             self.annotator_input, self.annotator_apply, self.annotator_export
         )
         return column(first_row, second_row)
 
-    def setup_widgets(self):
+    def _setup_widgets(self):
         """
         Create annotator widgets and assign Python callbacks.
         """
         from bokeh.models import TextInput, Button
         from bokeh.events import ButtonClick
 
-        super().setup_widgets()
+        super()._setup_widgets()
 
         self.annotator_input = TextInput(title="Label")
         self.annotator_apply = Button(
@@ -301,31 +342,38 @@ class BokehCorpusAnnotator(BokehForLabeledText):
         )
 
         def apply():
-            # a callback on clicking self.annotator_apply
-            # update labels in the source
+            """
+            A callback on clicking the 'self.annotator_apply' button.
+
+            Update labels in the source.
+            """
             label = self.annotator_input.value
-            selected_idx = self.source.selected.indices
+            selected_idx = self.sources["raw"].selected.indices
             if not selected_idx:
                 logger.warn("Did not select any data points.")
                 return
-            example_old = self.df_working.at[selected_idx[0], "label"]
-            self.df_working.at[selected_idx, "label"] = label
-            example_new = self.df_working.at[selected_idx[0], "label"]
+            example_old = self.dfs["raw"].at[selected_idx[0], "label"]
+            self.dfs["raw"].at[selected_idx, "label"] = label
+            example_new = self.dfs["raw"].at[selected_idx[0], "label"]
             logger.good(f"Updated DataFrame, e.g. {example_old} -> {example_new}")
 
-            self.reset_source()
+            self.update_source()
             self.plot()
             logger.good(f"Updated annotator plot at {current_time()}")
 
         def export():
-            # a callback on clicking self.annotator_export
+            """
+            A callback on clicking the 'self.annotator_export' button.
+
+            Saves the dataframe to a pickle.
+            """
             from dill import dump
             from datetime import datetime
 
             timestamp = datetime.now().strftime("%Y%m%d%H%M%S")
             filename = f"bokeh-annotated-df-{timestamp}.pkl"
             with open(filename, "wb") as f:
-                dump(self.df_working, f)
+                dump(self.dfs["raw"], f)
             logger.good(f"Saved DataFrame to {filename}")
 
         self.annotator_apply.on_event(ButtonClick, apply)
@@ -334,10 +382,13 @@ class BokehCorpusAnnotator(BokehForLabeledText):
     def plot(self):
         """
         Re-plot with the new labels.
+
+        Overrides the parent method.
+        Determines the label->color mapping dynamically.
         """
         from bokeh.transform import factor_cmap
 
-        all_labels = sorted(set(self.df_working["label"].values), reverse=True)
+        all_labels = sorted(set(self.dfs["raw"]["label"].values), reverse=True)
         assert len(all_labels) <= 20, "Too many labels to support"
         cmap = "Category10_10" if len(all_labels) <= 10 else "Category20_20"
 
@@ -346,8 +397,8 @@ class BokehCorpusAnnotator(BokehForLabeledText):
             y="y",
             color=factor_cmap("label", cmap, all_labels),
             legend_field="label",
-            source=self.source,
-            **self.plot_kwargs,
+            source=self.sources["raw"],
+            **self.glyph_kwargs["raw"],
         )
 
 
@@ -358,33 +409,30 @@ class BokehMarginExplorer(BokehCorpusExplorer):
     Currently not considering multi-label scenarios.
     """
 
-    BACKGROUND_GLYPH_KWARGS = {
-        "color": "gainsboro",
-        "line_alpha": 0.3,
-        "fill_alpha": 0.0,
+    DATA_KEY_TO_KWARGS = {
+        "raw": {
+            "constant": {"color": "gainsboro", "line_alpha": 0.5, "fill_alpha": 0.0},
+            "search": {"size": ("size", 10, 5, 7)},
+        }
     }
 
-    def __init__(self, df_raw, label_col_a, label_col_b, **kwargs):
+    def __init__(self, df_dict, label_col_a, label_col_b, **kwargs):
         """
         On top of the requirements of the parent class,
         the input dataframe should contain:
 
         (1) label_col_a and label_col_b for "label margins".
         """
-        super().__init__(df_raw, **kwargs)
-
-        for _key in ["text", label_col_a, label_col_b, "x", "y"]:
-            assert _key in self.df_raw.columns
         self.label_col_a = label_col_a
         self.label_col_b = label_col_b
+        super().__init__(df_dict, **kwargs)
 
-    def _activate_search_on_corpus(self):
-        """
-        Overriding the parent method, because there will be labels.
-        """
-        self.background_kwargs = self.activate_search(
-            self.source, self.background_kwargs, altered_param=("size", 6, 1, 3)
-        )
+    def _setup_dfs(self, df_dict):
+        """Extending from the parent method."""
+        for _key in [self.label_col_a, self.label_col_b]:
+            assert _key in df_dict["raw"].columns
+
+        super()._setup_dfs(df_dict)
 
     def plot(self, label, **kwargs):
         """
@@ -393,23 +441,26 @@ class BokehMarginExplorer(BokehCorpusExplorer):
 
         # prepare plot settings
         axes = ("x", "y")
-        eff_kwargs = self.background_kwargs.copy()
+        eff_kwargs = self.glyph_kwargs["raw"].copy()
         eff_kwargs.update(kwargs)
         eff_kwargs["legend_label"] = f"{label}"
 
         # create agreement/increment/decrement subsets
-        col_a_pos = np.where(self.df_raw[self.label_col_a] == label)[0].tolist()
-        col_a_neg = np.where(self.df_raw[self.label_col_a] != label)[0].tolist()
-        col_b_pos = np.where(self.df_raw[self.label_col_b] == label)[0].tolist()
-        col_b_neg = np.where(self.df_raw[self.label_col_b] != label)[0].tolist()
+        col_a_pos = np.where(self.dfs["raw"][self.label_col_a] == label)[0].tolist()
+        col_a_neg = np.where(self.dfs["raw"][self.label_col_a] != label)[0].tolist()
+        col_b_pos = np.where(self.dfs["raw"][self.label_col_b] == label)[0].tolist()
+        col_b_neg = np.where(self.dfs["raw"][self.label_col_b] != label)[0].tolist()
         agreement_view = CDSView(
-            source=self.source, filters=[IndexFilter(col_a_pos), IndexFilter(col_b_pos)]
+            source=self.sources["raw"],
+            filters=[IndexFilter(col_a_pos), IndexFilter(col_b_pos)],
         )
         increment_view = CDSView(
-            source=self.source, filters=[IndexFilter(col_a_neg), IndexFilter(col_b_pos)]
+            source=self.sources["raw"],
+            filters=[IndexFilter(col_a_neg), IndexFilter(col_b_pos)],
         )
         decrement_view = CDSView(
-            source=self.source, filters=[IndexFilter(col_a_pos), IndexFilter(col_b_neg)]
+            source=self.sources["raw"],
+            filters=[IndexFilter(col_a_pos), IndexFilter(col_b_neg)],
         )
 
         to_plot = [
@@ -422,7 +473,7 @@ class BokehMarginExplorer(BokehCorpusExplorer):
         for _dict in to_plot:
             _view = _dict["view"]
             _marker = _dict["marker"]
-            _marker(*axes, source=self.source, view=_view, **eff_kwargs)
+            _marker(*axes, source=self.sources["raw"], view=_view, **eff_kwargs)
 
 
 class BokehSnorkelExplorer(BokehCorpusExplorer):
@@ -430,128 +481,152 @@ class BokehSnorkelExplorer(BokehCorpusExplorer):
     Plot text data points along with labeling function outputs.
     """
 
-    BACKGROUND_GLYPH_KWARGS = {"line_alpha": 0.6, "fill_alpha": 0.0, "size": 7}
+    DATA_KEY_TO_KWARGS = {
+        "raw": {
+            "constant": {"line_alpha": 1.0, "color": "gainsboro"},
+            "search": {
+                "size": ("size", 10, 5, 7),
+                "fill_alpha": ("fill_alpha", 0.4, 0.05, 0.2),
+            },
+        },
+        "labeled": {
+            "constant": {"line_alpha": 1.0, "fill_alpha": 0.0},
+            "search": {"size": ("size", 10, 5, 7)},
+        },
+    }
 
-    def __init__(self, df_raw, df_labeled, **kwargs):
+    def __init__(self, df_dict, **kwargs):
         """
         On top of the requirements of the parent class,
         the df_labeled input dataframe should contain:
 
         (1) a "label" column for "ground truths".
         """
-        super().__init__(df_raw, **kwargs)
+        super().__init__(df_dict, **kwargs)
 
-        # add 'label' column to df_raw
-        if not "label" in self.df_raw.columns:
-            self.df_raw["label"] = module_config.ABSTAIN_DECODED
-
-        # prepare plot-ready dataframe for dev set
-        for _key in ["text", "label", "x", "y"]:
-            assert _key in df_labeled.columns
-        self.df_labeled = df_labeled.copy()
-
-        # initialize a list of LFs to enable toggles
+        # initialize a list to keep track of plotted LFs
         self.lfs = []
+        self.palette = Category20[20]
 
-    def _activate_search_on_corpus(self):
-        """
-        Overriding the parent method, because there will be a labeled dev set.
-        """
-        self.background_kwargs = self.activate_search(
-            self.source, self.background_kwargs, altered_param=("size", 10, 5, 7)
-        )
-        self.background_kwargs = self.activate_search(
-            self.source,
-            self.background_kwargs,
-            altered_param=("fill_alpha", 0.6, 0.0, 0.3),
-        )
+    def _setup_dfs(self, df_dict):
+        """Extending from the parent method."""
+        super()._setup_dfs(df_dict)
 
-    def plot(self, lf, L_raw=None, L_labeled=None, include=("C", "I", "M"), **kwargs):
+        assert "label" in self.dfs["labeled"].columns
+        if not "label" in self.dfs["raw"].columns:
+            self.dfs["raw"]["label"] = module_config.ABSTAIN_DECODED
+
+    def plot_lf(
+        self, lf, L_raw=None, L_labeled=None, include=("C", "I", "M"), **kwargs
+    ):
         """
         Plot a single labeling function.
+
         :param lf: labeling function decorated by @hover.utils.snorkel_helper.labeling_function()
+        :param L_raw: labeling function predictions, in decoded labels, on the raw df.
+        :param L_labeled: labeling function predictions, in decoded labels, on the labeled df.
+        :param include: subsets to show, which can be correct(C)/incorrect(I)/missed(M)/hit(H).
         """
         # keep track of added LF
         self.lfs.append(lf)
 
         # calculate predicted labels if not provided
         if L_raw is None:
-            L_raw = self.df_raw.apply(lf.row_to_label, axis=1).values
+            L_raw = self.dfs["raw"].apply(lf, axis=1).values
         if L_labeled is None:
-            L_labeled = self.df_labeled.apply(lf.row_to_label, axis=1).values
+            L_labeled = self.dfs["labeled"].apply(lf, axis=1).values
 
         # prepare plot settings
         axes = ("x", "y")
-        decoded_targets = [lf.label_decoder[_target] for _target in lf.targets]
-        eff_kwargs = self.background_kwargs.copy()
-        eff_kwargs.update(kwargs)
-        eff_kwargs["legend_label"] = f"{', '.join(decoded_targets)} | {lf.name}"
+        legend_label = f"{', '.join(lf.targets)} | {lf.name}"
+        color = self.palette[len(self.lfs) - 1]
+
+        raw_glyph_kwargs = self.glyph_kwargs["raw"].copy()
+        raw_glyph_kwargs["legend_label"] = legend_label
+        raw_glyph_kwargs["color"] = color
+        raw_glyph_kwargs.update(kwargs)
+
+        labeled_glyph_kwargs = self.glyph_kwargs["labeled"].copy()
+        labeled_glyph_kwargs["legend_label"] = legend_label
+        labeled_glyph_kwargs["color"] = color
+        labeled_glyph_kwargs.update(kwargs)
 
         # create correct/incorrect/missed/hit subsets
         to_plot = []
         if "C" in include:
             to_plot.append(
-                {"view": self._view_correct(L_labeled), "marker": self.figure.square}
+                {
+                    "view": self._view_correct(L_labeled),
+                    "marker": self.figure.square,
+                    "kwargs": labeled_glyph_kwargs,
+                }
             )
         if "I" in include:
             to_plot.append(
-                {"view": self._view_incorrect(L_labeled), "marker": self.figure.x}
+                {
+                    "view": self._view_incorrect(L_labeled),
+                    "marker": self.figure.x,
+                    "kwargs": labeled_glyph_kwargs,
+                }
             )
         if "M" in include:
             to_plot.append(
                 {
                     "view": self._view_missed(L_labeled, lf.targets),
                     "marker": self.figure.cross,
+                    "kwargs": labeled_glyph_kwargs,
                 }
             )
         if "H" in include:
             to_plot.append(
-                {"view": self._view_hit(L_raw), "marker": self.figure.circle}
+                {
+                    "view": self._view_hit(L_raw),
+                    "marker": self.figure.circle,
+                    "kwargs": raw_glyph_kwargs,
+                }
             )
 
         # plot created subsets
         for _dict in to_plot:
             _view = _dict["view"]
             _marker = _dict["marker"]
-            _marker(*axes, source=self.source, view=_view, **eff_kwargs)
+            _kwargs = _dict["kwargs"]
+            _marker(*axes, source=_view.source, view=_view, **_kwargs)
 
     def _view_correct(self, L_labeled):
         """
         Determine the subset correctly labeled by a labeling function.
         """
-
-        indices_raw = self.df_labeled["label"].values == L_labeled
-        indices = np.where(indices_raw)[0].tolist()
-        view = CDSView(source=self.source, filters=[IndexFilter(indices)])
+        agreed = self.dfs["labeled"]["label"].values == L_labeled
+        attempted = L_labeled != module_config.ABSTAIN_DECODED
+        indices = np.where(np.multiply(agreed, attempted))[0].tolist()
+        view = CDSView(source=self.sources["labeled"], filters=[IndexFilter(indices)])
         return view
 
     def _view_incorrect(self, L_labeled):
         """
         Determine the subset incorrectly labeled by a labeling function.
         """
-        disagreed = self.df_labeled["label"].values != L_labeled
+        disagreed = self.dfs["labeled"]["label"].values != L_labeled
         attempted = L_labeled != module_config.ABSTAIN_DECODED
-        indices_raw = np.multiply(disagreed, attempted)
-        indices = np.where(indices_raw)[0].tolist()
-        view = CDSView(source=self.source, filters=[IndexFilter(indices)])
+        indices = np.where(np.multiply(disagreed, attempted))[0].tolist()
+        view = CDSView(source=self.sources["labeled"], filters=[IndexFilter(indices)])
         return view
 
     def _view_missed(self, L_labeled, targets):
         """
         Determine the subset missed by a labeling function.
         """
-        targetable = np.isin(self.df_labeled["label"], targets)
+        targetable = np.isin(self.dfs["labeled"]["label"], targets)
         abstained = L_labeled == module_config.ABSTAIN_DECODED
-        indices_raw = np.multiply(targetable, abstained)
-        indices = np.where(indices_raw)[0].tolist()
-        view = CDSView(source=self.source, filters=[IndexFilter(indices)])
+        indices = np.where(np.multiply(targetable, abstained))[0].tolist()
+        view = CDSView(source=self.sources["labeled"], filters=[IndexFilter(indices)])
         return view
 
     def _view_hit(self, L_raw):
         """
         Determine the subset hit by a labeling function.
         """
-        indices_raw = L_raw != module_config.ABSTAIN_DECODED
-        indices = np.where(indices_raw)[0].tolist()
-        view = CDSView(source=self.source, filters=[IndexFilter(indices)])
+        indices = np.where(L_raw != module_config.ABSTAIN_DECODED)[0].tolist()
+        view = CDSView(source=self.sources["raw"], filters=[IndexFilter(indices)])
         return view
