@@ -1,7 +1,6 @@
 import torch
 import torch.nn.functional as F
 from datetime import datetime
-from hover.utils.torch_helper import vector_dataloader, one_hot, label_smoothing
 from hover.utils.metrics import classification_accuracy
 from wasabi import msg as logger
 from sklearn.metrics import confusion_matrix
@@ -12,6 +11,7 @@ import numpy as np
 def create_vector_net_from_module(specific_class, model_module_name, labels):
     """
     Create a TextVectorNet model, or of its child class.
+
     :param specific_class: TextVectorNet or its child class.
     :type specific_class: class
     :param model_module_name: path to a local Python module in the working directory whose __init__.py file contains a get_vectorizer() callable, get_architecture() callable, and a get_state_dict_path() callable.
@@ -35,7 +35,9 @@ def create_vector_net_from_module(specific_class, model_module_name, labels):
 
 
 class VectorNet(object):
-    """Simple transfer learning model: a user-supplied vectorizer followed by a neural net.
+
+    """
+    Simple transfer learning model: a user-supplied vectorizer followed by a neural net.
     This is a parent class whose children may use different training schemes.
     Please refer to hover.utils.torch_helper.VectorDataset and vector_dataloader for more info.
     """
@@ -203,7 +205,7 @@ class VectorNet(object):
         Supports flexible args and kwargs for child classes that may implement self.train() and self.train_batch() differently.
         """
         self.adjust_optimizer_params()
-        for batch_idx, (loaded_input, loaded_output, index) in enumerate(train_loader):
+        for batch_idx, (loaded_input, loaded_output, _) in enumerate(train_loader):
             self._dynamic_params["batch"] = batch_idx + 1
             self.train_batch(loaded_input, loaded_output, *args, **kwargs)
 
